
#include <sstream>
using namespace std;

#include <libsaria/audio.h>
#include "audio.h"

static bool get_duration(gint64 &duration)
{
	GstFormat fmt = GST_FORMAT_TIME;
	return gst_element_query_duration(GST_ELEMENT(player),
					  &fmt,
					  &duration);
}

static bool get_position(gint64 &position)
{
	GstFormat fmt = GST_FORMAT_TIME;
	return gst_element_query_position(GST_ELEMENT(player),
					  &fmt,
					  &position);
}

static string to_string(gint64 time)
{
	stringstream stream;
	int minutes, seconds;

	if (time == 0)
		return "";

	time = time / GST_SECOND;
	minutes = time / 60;
	seconds = time - (minutes * 60);
	stream << minutes << ":";
	if (seconds < 10)
		stream << "0";
	stream << seconds;
	return stream.str();
}

namespace libsaria
{

	/* Seek to this many seconds into the song */
	void audio::seek_to(double pos)
	{
		gst_element_seek_simple(GST_ELEMENT(player),
					GST_FORMAT_TIME,
					GST_SEEK_FLAG_FLUSH,
					pos);
	}

	/*
	 * Use to seek forward or backward in a song
	 * by dt seconds.
	 */
	bool audio::seek(int dt)
	{
		bool ret;
		gint64 pos;

		ret = get_position(pos);
		if (ret == true) {
			/* Convert seconds to nano-seconds */
			pos += (dt * GST_SECOND);
			if (pos < 0)
				pos = 0;
			seek_to(pos);
		}
		return ret;
	}

	gint64 audio::duration()
	{
		gint64 duration;
		if (!get_duration(duration))
			return 0;
		if (duration < 0)
			duration = 0;
		return duration;
	}

	string audio::durstr()
	{
		return to_string(duration());
	}

	gint64 audio::position()
	{
		gint64 position;
		if (!get_position(position))
			return 0;
		return position;
	}

	string audio::posstr()
	{
		return to_string(position());
	}

};
