
#include <libsaria/index.h>
#include <libsaria/format.h>
#include <libsaria/callback.h>
#include "index.h"

set<sid_t> results;
set<string> *cur_terms;
static bool filtered = false;

/*
 * Thanks to: http://stackoverflow.com/questions/1773526/in-place-c-set-intersection
 * for the set intersection algorithm!
 */
static void inplace_intersect(set<sid_t> *ids)
{
	set<sid_t>::iterator it1 = results.begin();
	set<sid_t>::iterator it2 = ids->begin();

	while ( (it1 != results.end()) && (it2 != ids->end()) ) {
		if (*it1 < *it2)
			results.erase(it1++);
		else if (*it2 < *it1)
			it2++;
		else { /* *it1 == *it2 */
			it1++;
			it2++;
		}
	}

	/* Remove everything in results that wasn't in inodes */
	results.erase(it1, results.end());

}

static inline void search_index(const string &term,
				map<string, set<sid_t> > *index,
				set<ino_t> *found)
{
	map<string, set<sid_t> >::iterator it;
	it = index->find(term);
	if (it != index->end())
		found->insert(it->second.begin(), it->second.end());
}

static inline void search_indexes(const string &term, set<ino_t> *found)
{
	search_index(term, &artist_index, found);
	search_index(term, &album_index,  found);
	search_index(term, &title_index,  found);
}

static void do_filter(set<string> *terms)
{
	set<ino_t> found;
	set<string>::iterator it;

	for (it = terms->begin(); it != terms->end(); it++) {
		found.clear();
		search_indexes(*it, &found);
		/*
		 * Key not found means we don't need to filter anymore
		 * just clear the results set and return.
		 */
		if (found.size() == 0) {
			results.clear();
			return;
		}
		/*
		 * This is the first result, so the result set is empty.
		 * Taking an intersection will always give us an empty set.
		 */
		if (it == terms->begin())
			results = found;
		else
			inplace_intersect(&found);
	}
}

static void run_filter()
{
	results.clear();

	if (cur_terms == NULL || cur_terms->size() == 0)
		filtered = false;
	else {
		do_filter(cur_terms);
		filtered = true;
	}
	trigger_callback(REFILTER);
}

namespace libsaria
{

	void index::filter(string &search)
	{
		cur_terms = format_text(search);
		run_filter();
	}

	void index::refresh()
	{
		run_filter();
	}

	bool index::is_visible(sid_t &songid)
	{
		set<sid_t>::iterator it;

		if (filtered == false)
			return true;
		it = results.find(songid);
		return it != results.end();
	}

	bool index::is_filtered()
	{
		return filtered;
	}

	unsigned int index::size()
	{
		return results.size();
	}
};
