
#include <libsaria/track.h>
#include <libsaria/index.h>
#include <libsaria/format.h>
#include "index.h"

#include <map>
#include <set>
#include <string>
#include <stdlib.h>
using namespace std;

map<string, set<sid_t> > artist_index;
map<string, set<sid_t> > album_index;
map<string, set<sid_t> > title_index;

static map<string, set<string> > substr_cache;
unsigned int hits;

static set<string> *gen_substrs(const string &word)
{
	string substr, key;
	set<string> substrs;
	pair< map<string, set<string> >::iterator, bool > ret;

	for (unsigned int i = 0; i < word.size(); i++) {
		substr = word.substr(i, word.size());
		for (unsigned int j = 1; j < substr.size() + 1; j++) {
			key = substr.substr(0, j);
			substrs.insert(key);
		}
	}

	ret = substr_cache.insert( pair<string, set<string> >(word, substrs));
	return &(ret.first->second);
}

static set<string> *find_substrs(const string &word)
{
	map<string, set<string> >::iterator it;
	it = substr_cache.find(word);
	if (it == substr_cache.end())
		return gen_substrs(word);
	else {
		hits++;
		return &(it->second);
	}
}

static void add_to_index(sid_t &songid, string key,
			 map<string, set<sid_t> >*index)
{
	map<string, set<sid_t> >::iterator it;

	it = index->find(key);
	if (it != index->end())
		it->second.insert(songid);
	else {
		set<sid_t> ids;
		ids.insert(songid);
		index->insert(pair<string, set<sid_t> >(key, ids));
	}
}

static void index_word(sid_t &songid, const string &word,
		       map<string, set<sid_t> > *index)
{
	set<string> *substrs = find_substrs(word);
	set<string>::iterator it;
	for (it = substrs->begin(); it != substrs->end(); it++)
		add_to_index(songid, *it, index);
}

static void index_tag(sid_t &songid, const string &tag,
		      map<string, set<sid_t> >*index)
{
	set<string> *word_set;
	set<string>::iterator it;

	word_set = libsaria::format_text(tag);
	for (it = word_set->begin(); it != word_set->end(); it++)
		index_word(songid, *it, index);
}

namespace libsaria
{

	void index::add_track(libsaria::Track *track)
	{
		sid_t songid = track->get_songid();
		index_tag(songid, track->get_artist(), &artist_index);
		index_tag(songid, track->get_album(),  &album_index);
		index_tag(songid, track->get_title(),  &title_index);
	}

	void index::print_stats()
	{
		println("Index cache hits: %u size: %u", hits, substr_cache.size());
	}

	sid_t index::rand_id()
	{
		set<sid_t>::iterator it = results.begin();
		unsigned int index = rand() % results.size();
		println("results.size()? %d", results.size());

		for (unsigned int i = 0; i < index; i++)
			it++;
		return *it;
	}
}
