
#include <list>
#include <string>
using namespace std;

#include <libsaria/model.h>
#include <libsaria/callback.h>
#include <libsaria/library.h>
#include "library.h"

list<libsaria::LibraryPath> path_list;

static void do_update(libsaria::LibraryPath *path)
{
	path->validate();
	path->update();
}

namespace libsaria
{

	void library::for_each_path(void (*info_func)(struct library::PathInfo &))
	{
		list<LibraryPath>::iterator it;
		for (it = path_list.begin(); it != path_list.end(); it++)
			it->get_info(info_func);
	}

	void library::update()
	{
		pathiter::reset();
		while (!pathiter::end())
			do_update(pathiter::next());
	}

	void library::update_path(string dir)
	{
		LibraryPath *path;

		pathiter::reset();
		while (!pathiter::end()) {
			path = pathiter::next();
			if (path->get_path() == dir)
				do_update(path);
		}
	}

	void library::refresh()
	{
		trigger_callback(LIBRARY_REFRESH);
	}

	void library::add_path(string dir)
	{
		path_list.push_back(LibraryPath(dir));
		path_list.back().update();
	}

	void library::remove_path(string dir)
	{
		list<LibraryPath>::iterator it;

		for (it = path_list.begin(); it != path_list.end(); it++) {
			if (it->get_path() == dir) {
				it->prepare_for_removal();
				path_list.erase(it);
				save();
				refresh();
				return;
			}
		}
	}


}; /* namespace: libsaria */
