
#include <libsaria/library.h>
#include <libsaria/index.h>
#include <libsaria/prefs.h>
#include "list.h"

#include <stdlib.h>

/*
 * I initialize the iterator to the end of the list so
 * the code that handling looping will reset the iterator
 * to the beginning of the list
 */
static list<libsaria::Track *>::iterator current = track_list.end();

static inline void incr_iter()
{
	current++;
	if (current == track_list.end())
		current = track_list.begin();
}

static void find_id(sid_t songid)
{
	for (current = track_list.begin(); current != track_list.end(); current++) {
		if ((*current)->get_songid() == songid)
			return;
	}
}

static void random_next()
{
	if (libsaria::index::is_filtered())
		find_id( libsaria::index::rand_id() );
	else {
		unsigned int n = rand() % track_list.size();
		for (unsigned int i = 0; i < n; i++)
			incr_iter();
	}
}

static void sequential_next()
{
	sid_t songid;
	do {
		incr_iter();
		songid = (*current)->get_songid();
	} while(!libsaria::index::is_visible(songid));
}

static string pick_next()
{
	if (libsaria::prefs::get_bool("random") == true)
		random_next();
	else
		sequential_next();
	return (*current)->get_filepath();
}

namespace libsaria
{

	string library::next_file()
	{
		string filepath = "";

		if (size() == 0)
			return filepath;

		do
			filepath = pick_next();
		while (filepath == "");

		return filepath;
	}

	string library::find_filepath(sid_t &songid)
	{
		find_id(songid);
		return (*current)->get_filepath();
	}

} /* Namespace: libsaria */
