
#include <libsaria/path.h>
#include <libsaria/libpath.h>

#define MAX_VALIDATE 250

class ValidateTask : public IdleTask
{
	private:
		libsaria::LibraryPath *lib_path;
		list<libsaria::Track *> tracks;

	public:
		ValidateTask(libsaria::LibraryPath *path)
		{
			lib_path = path;
		}
		~ValidateTask() {};

		unsigned int add_track(libsaria::Track *track)
		{
			tracks.push_back(track);
			return tracks.size();
		}

		void run_task();
};

void ValidateTask::run_task()
{
	list<libsaria::Track *>::iterator it;
	for (it = tracks.begin(); it != tracks.end(); it++) {
		lib_path->validate_track(*it);
	}
}

namespace libsaria
{

	void LibraryPath::validate_track(libsaria::Track *track)
	{
		string filepath = track->get_filepath();
		if (lookup_songid(filepath) == track->get_songid())
			return;
		track->do_cleanup();
		delete track;
		file_list.remove(track);
	}

	void LibraryPath::validate()
	{
		list<libsaria::Track *>::iterator it;
		ValidateTask *validate = new ValidateTask(this);
		println("Validating");

		for (it = file_list.begin(); it != file_list.end(); it++) {
			if (validate->add_track(*it) > MAX_VALIDATE) {
				validate->queue();
				validate = new ValidateTask(this);
			}
		}
		validate->queue();
	}

} /* Namespace: libsaria */
