
#include <libsaria/libsaria.h>
#include <libsaria/callback.h>
#include <libsaria/print.h>
#include <ocarina/ocarina.h>
#include <ocarina/button.h>
#include <ocarina/library.h>
#include <ocarina/queue.h>
#include <ocarina/settings.h>
#include <ocarina/footer.h>

static guint signalid;
static gpointer object;

static void handle_callback(gpointer obj, callback_t type, gpointer data)
{
	libsaria::Track *current;
	println("type? %d", type);

	switch (type) {
	case PLAY:
		hide_button_list(PLAY_BUTTON_LIST);
		show_button_list(PAUSE_BUTTON_LIST);
		break;
	case PAUSE:
		hide_button_list(PAUSE_BUTTON_LIST);
		show_button_list(PLAY_BUTTON_LIST);
		break;
	case IDLE_TASK_QUEUED:
		ocarina::idle_add();
		break;
	case LIBRARY_REFRESH:
		ocarina::library::refresh();
		library_settings_refresh();
		break;
	case QUEUE_REFRESH:
		ocarina::queue::refresh();
		break;
	case REFILTER:
		ocarina::library::refilter();
		break;
	case TRACK_LOADED:
		current = libsaria::current_track();
		change_footer(current);
		ocarina::library::scroll_to(current);
		break;
	default:
		break;
	}
}

static void cb_lock(callback_t type)
{
	switch(type) {
	case TRACK_LOADED:
		gdk_threads_enter();
	default:
		break;
	}
}

static void cb_unlock(callback_t type)
{
	switch(type) {
	case TRACK_LOADED:
		gdk_threads_leave();
	default:
		break;
	};
}

static void cb_handler(callback_t type)
{
	cb_lock(type);
	g_signal_emit_by_name(G_OBJECT(object), "ocarina-callback", type);
	cb_unlock(type);
}

void setup_callbacks()
{
	println("Ocarina setting up callbacks");
	object = g_object_new(G_TYPE_OBJECT, NULL);
	signalid = g_signal_new("ocarina-callback",
				G_TYPE_OBJECT,
				G_SIGNAL_RUN_LAST,
				0,
				NULL,
				NULL,
				NULL,
				G_TYPE_NONE,
				1,
				G_TYPE_UINT);
	g_signal_connect(G_OBJECT(object),
			 "ocarina-callback",
			 G_CALLBACK(handle_callback),
			 NULL);
	set_cb_handler(cb_handler);
}
