
#include <libsaria/track.h>
#include <ocarina/gtk.h>
#include "footer.h"

#include <list>
using namespace std;

struct NowPlayingWidgets {
	GtkWidget *title;
	GtkWidget *artist;
	GtkWidget *album;
};

static list<NowPlayingWidgets> nowplaying_widgets;

static void set_label(GtkWidget *label, string text, string size)
{
	char *escaped = g_markup_escape_text(text.c_str(), -1);
	string markup  = "<span size='" + size + "'>" + escaped + "</span>";
	gtk_label_set_markup(GTK_LABEL(label), markup.c_str());
	g_free(escaped);
}

static void set_now_playing(struct NowPlayingWidgets *widgets, string title,
			    string artist, string album)
{
	if (artist != "")
		artist = "by " + artist;

	if (album != "")
		album = "from " + album;

	set_label(widgets->title,  title,  "xx-large");
	set_label(widgets->artist, artist, "x-large");
	set_label(widgets->album,  album,  "x-large");
}

static GtkWidget *align(GtkWidget *label)
{
	GtkWidget *alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	container_add(alignment, label);
	return alignment;
}

static GtkWidget *setup_window(GtkWidget *box)
{
	GtkWidget *viewport = gtk_viewport_new(NULL, NULL);
	GtkWidget *window   = gtk_scrolled_window_new(NULL, NULL);

	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(window),
				       GTK_POLICY_AUTOMATIC,
				       GTK_POLICY_AUTOMATIC);
	gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewport), GTK_SHADOW_NONE);
	container_add(viewport, box);
	container_add(window, viewport);

	return window;
}

GtkWidget *get_nowplaying()
{
	struct NowPlayingWidgets widgets;
	GtkWidget *box    = gtk_vbox_new(FALSE, 0);
	GtkWidget *window = setup_window(box);

	widgets.title  = gtk_label_new("");
	widgets.artist = gtk_label_new("");
	widgets.album  = gtk_label_new("");

	set_now_playing(&widgets, "", "", "");

	box_pack_start(box, align(widgets.title),  TRUE, FALSE, 0);
	box_pack_start(box, align(widgets.artist), TRUE, FALSE, 0);
	box_pack_start(box, align(widgets.album),  TRUE, FALSE, 0);

	nowplaying_widgets.push_back(widgets);
	gtk_widget_show_all(window);
	return window;
}

/*
 * This takes some explanation, so...
 * During startup, all the labels are set to the empty string.  This
 * causes them to initially be rendered blank.  This normally isn't
 * a problem BUT the three widgets placed on the settings tab don't have
 * a call to gtk_widget_show() once the page is added to the notebook and
 * for some reason switching tabs isn't enough to realize the widgets.
 * As a result, only the very beginning of the widget gets updated (as in,
 * you could see half of a "T" if you start ocarina, click "next" and then
 * switch to the settings tab).  Realizing just one of the widgets before
 * trying to use them seems to fix the problem, so that's what I'm doing
 * here.
 */
void nowplaying_init()
{
	list<struct NowPlayingWidgets>::iterator it;
	for (it = nowplaying_widgets.begin(); it != nowplaying_widgets.end(); it++)
		gtk_widget_realize(it->title);
}

void set_now_playing(libsaria::Track *track)
{
	list<struct NowPlayingWidgets>::iterator it;

	for (it = nowplaying_widgets.begin(); it != nowplaying_widgets.end(); it++) {
		set_now_playing(&(*it),
				track->get_title(),
				track->get_artist(),
				track->get_album());
	};
}
