
#include <ocarina/gtk.h>
#include <libsaria/audio.h>
#include <libsaria/print.h>
#include "footer.h"

#include <list>
using namespace std;

struct ProgressWidgets {
	GtkWidget *position;
	GtkWidget *duration;
	GtkWidget *progress;
};

static list<ProgressWidgets> widget_list;

static gboolean update_progress(struct ProgressWidgets *widgets)
{
	gint64 pos = libsaria::audio::position();
	gint64 dur = libsaria::audio::duration() + 1;

	gtk_label_set_text(GTK_LABEL(widgets->position), libsaria::audio::posstr().c_str());
	gtk_label_set_text(GTK_LABEL(widgets->duration), libsaria::audio::durstr().c_str());

	/*
	 * This happens when gstreamer's "about-to-finish" signal
	 * is emitted and we queue up a shorter song before the
	 * pipeline finishes.
	 */
	if (pos > dur)
		pos = 0;
	gtk_range_set_range(GTK_RANGE(widgets->progress), 0, dur);
	gtk_range_set_value(GTK_RANGE(widgets->progress), pos);
	return TRUE;
}

static gboolean update_progress_widgets(gpointer data)
{
	list<ProgressWidgets>::iterator it;
	for (it = widget_list.begin(); it != widget_list.end(); it++)
		update_progress(&(*it));
	return TRUE;
}

void progress_init()
{
	gtk_timeout_add(100, update_progress_widgets, NULL);
}

static void slider_changed(GtkWidget *slider, GtkScrollType scroll,
			   gdouble value, gpointer data)
{
	libsaria::audio::seek_to(value);
}

static GtkWidget *make_scale()
{
	GtkWidget *scale = gtk_hscale_new_with_range(0, 1, 1000000000);
	gtk_scale_set_draw_value(GTK_SCALE(scale), FALSE);
	GTK_CONNECT(scale, "change-value", slider_changed, NULL);
	return scale;
}

GtkWidget *get_progress()
{
	struct ProgressWidgets widgets;
	GtkWidget *box = gtk_hbox_new(FALSE, 0);
	widgets.position = gtk_label_new("");
	widgets.duration = gtk_label_new("");
	widgets.progress = make_scale();

	box_pack_start(box, widgets.position, FALSE, FALSE, 0);
	box_pack_start(box, widgets.progress,  TRUE,  TRUE, 0);
	box_pack_start(box, widgets.duration, FALSE, FALSE, 0);

	gtk_widget_show_all(box);
	widget_list.push_back(widgets);
	return box;
}
