
#include <sstream>
#include <map>
using namespace std;

#include <ocarina/settings.h>
#include <ocarina/chooser.h>
#include <ocarina/button.h>
#include <ocarina/gtk.h>

#include <libsaria/library.h>
#include <libsaria/libpath.h>
#include <libsaria/print.h>

/* This is a gtk vbox */
static GtkWidget *library_settings = NULL;
static map<GtkWidget *, string> widget_mapping;

static void on_click_add(GtkWidget *b, GdkEvent *e, gpointer d)
{
	string dir = ocarina::choose_dir();
	if (dir != "" ) {
		println("Scanning dir: " + dir);
		libsaria::library::add_path(dir);
	}
}

static void on_click_update(GtkWidget *b, GdkEvent *e, gpointer d)
{
	libsaria::library::update();
}

static void on_click_update_path(GtkWidget *b, GdkEvent *e, gpointer d)
{
	libsaria::library::update_path(widget_mapping[b]);
}

static void on_click_remove(GtkWidget *b, GdkEvent *e, gpointer d)
{
	libsaria::library::remove_path(widget_mapping[b]);
}

static GtkWidget *markup_label(string markup)
{
	GtkWidget *label = gtk_label_new("");
	gtk_label_set_markup(GTK_LABEL(label), markup.c_str());
	return label;
}

static GtkWidget *make_path_info(string path, unsigned int size)
{
	stringstream stream;
	GtkWidget *path_label, *size_label;
	GtkWidget *box = gtk_vbox_new(FALSE, 0);

	stream << "<span size='xx-large' weight='bold'>" << path << "</span>";
	path_label = markup_label(stream.str());
	stream.str("");

	stream << "<span size='large'>" << size << " files</span>";
	size_label = markup_label(stream.str());

	box_pack_start(box, path_label, TRUE, TRUE, 0);
	box_pack_start(box, size_label, TRUE, TRUE, 0);
	return box;
}

static void add_content(GtkWidget *panel, libsaria::LibraryPath *path)
{
	GtkWidget *content = gtk_hbox_new(FALSE, 0);
	GtkWidget *info    = make_path_info(path->get_path(), path->get_size());
	GtkWidget *update  = make_text_button(GTK_STOCK_REFRESH,
					      "Update Path",
					      on_click_update_path,
					      true);
	GtkWidget *remove  = make_text_button(GTK_STOCK_REMOVE,
					      "Remove Path",
					      on_click_remove,
					      true);

	box_pack_start(content,   info,  TRUE,  TRUE, 0);
	box_pack_start(content, update, FALSE, FALSE, 0);
	box_pack_start(content, remove, FALSE, FALSE, 0);

	box_pack_start(panel, content, TRUE, TRUE, 0);

	widget_mapping[update] = path->get_path();
	widget_mapping[remove] = path->get_path();
}

static void add_library_path(libsaria::LibraryPath *path)
{
	GtkWidget *panel = gtk_vbox_new(FALSE, 0);
	GtkWidget *sep   = gtk_hseparator_new();

	add_content(panel, path);

	box_pack_start(panel, sep, TRUE, TRUE, 0);
	box_pack_start(library_settings, panel, FALSE, FALSE, 0);
}

static GtkWidget *get_button_row()
{
	GtkWidget *button_row = gtk_hbox_new(FALSE, 0);
	GtkWidget *add_button = make_text_button(GTK_STOCK_ADD,
						 "Add to Library",
						 on_click_add,
						 true);
	GtkWidget *update_button = make_text_button(GTK_STOCK_REFRESH,
						 "Update Library",
						 on_click_update,
						 true);

	box_pack_end(button_row, add_button,    FALSE, FALSE, 0);
	box_pack_end(button_row, update_button, FALSE, FALSE, 0);

	return button_row;
}

void library_refresh_remove(GtkWidget *widget, gpointer data)
{
	gtk_widget_destroy(widget);
}

void library_settings_refresh()
{
	libsaria::LibraryPath *path;

	gtk_container_foreach(GTK_CONTAINER(library_settings), library_refresh_remove, NULL);
	widget_mapping.clear();

	libsaria::library::pathiter::reset();
	while (!libsaria::library::pathiter::end()) {
		path = libsaria::library::pathiter::next();
		add_library_path(path);
	}

	box_pack_start(library_settings, get_button_row(), FALSE, FALSE, 0);
	gtk_widget_show_all(library_settings);
}

void library_settings_init()
{
	library_settings = gtk_vbox_new(FALSE, 0);

	box_pack_start(library_settings, get_button_row(), FALSE, FALSE, 0);
	gtk_widget_show_all(library_settings);
	add_settings_page("Library", library_settings);
}
